# from pdb import set_trace
import os
import glob
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import re
from collections import defaultdict 

sns.set_palette('pastel')
folder = os.path.join(os.getcwd())
def save_ax(ax, name):
    ax.set_ylim(0,)
    fig = ax.figure
    fig.savefig(f'{name}.png')
    plt.close(fig)

csvs = glob.glob(os.path.join(folder, '{*}.csv'))
# print(csvs)
aggregated_csvs = defaultdict(list) 
# find the differently-seeded runs for each configuration
for csv in csvs:
    agg_csv = re.compile(r'seed:(\d+),').sub('', csv)
    aggregated_csvs[agg_csv].append(csv)

# aggregate csvs
agg_dfs = {}
for pair in aggregated_csvs.items():
    (agg_csv, fpaths) = pair
    dfs = []
    for csv in fpaths:
        df = pd.read_csv(csv, dialect='unix')
        # df['seed'] = int(re.compile(r'.*seed:(\d+).*').match(csv).group(1))
        dfs.append(df)
    df = pd.concat(dfs)
    agg_dfs[agg_csv] = df

    var_name = 'dataset'
    id_var = 'epoch'
    value_name = 'loss'
    value_map = {'lossTrain': 'train', 'lossValid': 'validation'}
    ax = sns.lineplot(
        x=id_var,
        y=value_name,
        hue=var_name,
        data=df.rename(columns=value_map).melt(var_name=var_name, value_name=value_name, id_vars=[id_var], value_vars=value_map.values())
    )
    fname = agg_csv.replace('.csv', f'-{value_name}')
    save_ax(ax, fname)

    metrics = ['accValid']
    for metric in metrics:
        ax = sns.lineplot(
            x=id_var,
            y=metric,
            data=df
        )
        fname = agg_csv.replace('.csv', f'-{metric}')
        save_ax(ax, fname)

experiments = {
    r'.*{taskPath:datasets-3\.yml,clip:.*,bestOf:100}.*': 'vanilla model (NSPS)',
    r'.*{h:64,taskPath:datasets-3\.yml,clip:.*,bestOf:100}.*': 'large vanilla',
    # r'.*randomHole:1.*3\.yml,clip:.*,bestOf:100.*': 'allow filling non-first holes',
    r'.*{taskPath:datasets-3\.yml,useTypes:1.*bestOf:100.*': 'types added as features',
    # r'.*randomHole:1.*3\.yml,useTypes:1.*bestOf:100.*': 'types + non-first holes',
    # r'.*maskBad:1.*3\.yml.*bestOf:100.*': 'mask out non-compiling programs',
}
desc_dfs = {}
for pair in experiments.items():
    (regex, desc) = pair
    rx = re.compile(regex)
    ks = [rx.match(x).group(0) for x in agg_dfs.keys() if rx.match(x)]
    if len(ks) == 1:
        agg_csv = ks[0]
    else:
        raise Exception(f"multiple matches found for regex {str(regex)}: {str(ks)}")
    desc_dfs[desc] = agg_dfs[agg_csv]

desc_name = 'model'
exp_dfs = []
for pair in desc_dfs.items():
    (desc, df) = pair
    df[desc_name] = desc
    exp_dfs.append(df)

ax = sns.lineplot(
    x=id_var,
    y='accValid',
    hue=desc_name,
    data=pd.concat(exp_dfs)
)
plt.legend(framealpha=0.2, loc='lower right')
plt.ylabel("accuracy")
fpath = os.path.join(folder, 'experiments')
save_ax(ax, fpath)
